     /**
      * \brief Flat File Parser : Database Logger
      *
      * All messages will be written to the file FFPLOG.
      *
      * \author Mihael Schmidt
      * \date   20.04.2010
      */


     H nomain

      *-------------------------------------------------------------------------
      *
      * (C) Copyleft 2011 Mihael Schmidt
      *
      * This file is part of Flat File Parser (FFP).
      *
      * FFP is free software: you can redistribute it and/or modify
      * it under the terms of the GNU General Public License as published
      * the Free Software Foundation, either version 3 of the License, or
      * any later version.
      *
      * FFP is distributed in the hope that it will be useful,
      * but WITHOUT ANY WARRANTY; without even the implied warranty of
      * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      * GNU General Public License for more details.
      *
      * You should have received a copy of the GNU General Public License
      * along with FFP.  If not, see <http://www.gnu.org/licenses/>.
      *
      *-------------------------------------------------------------------------
      
      
      *-------------------------------------------------------------------------
      * Files
      *-------------------------------------------------------------------------
     FFFPLOG    O    E             DISK    qualified usropn


      *-------------------------------------------------------------------------
      * Prototypes
      *-------------------------------------------------------------------------
      /copy 'logger_h.rpgle'
      /copy 'database_logger_h.rpgle'
      /copy 'libc_h.rpgle'


      *-------------------------------------------------------------------------
      * Constants
      *-------------------------------------------------------------------------
     D LOGGER_ID       C                   'ffp_logger_db'


      *-------------------------------------------------------------------------
      * Templates
      *-------------------------------------------------------------------------
      /copy 'logger_t.rpgle'


      *-------------------------------------------------------------------------
      * Procedures
      *-------------------------------------------------------------------------


     /**
      * \brief Create logger
      *
      * \param Userdata
      *
      * \info This logger implemenation needs no user data.
      */
     P ffp_logger_db_create...
     P                 B                   export
     D                 PI              *
     D   userdata                      *   const options(*nopass)
      *
     D logger          S               *
     D header          DS                  likeds(ffp_logger) based(logger)
      /free
       logger = %alloc(%size(ffp_logger));

       clear header;

       header.id = LOGGER_ID;
       header.proc_create = %paddr('ffp_logger_db_create');
       header.proc_log = %paddr('ffp_logger_db_log');
       header.proc_finalize = %paddr('ffp_logger_db_finalize');

       if (%parms() = 0 or userdata = *null);
         header.userdata = *null;
       else;
         header.userdata = %alloc(strlen(userdata) + 1); // speicher fr null reservieren
         memcpy(header.userdata : userdata : strlen(userdata) + 1); // null mit kopieren
       endif;

       return logger;
      /end-free
     P                 E


     /**
      * \brief Free resources
      *
      * Frees all previously allocated resources.
      *
      * \param Logger
      */
     P ffp_logger_db_finalize...
     P                 B                   export
     D                 PI
     D   logger                        *
      *
     D header          DS                  likeds(ffp_logger) based(logger)
      /free
        if (%open(FFPLOG));
         close(e) FFPLOG;
       endif;

       if (header.userdata <> *null);
         dealloc header.userdata;
       endif;

       dealloc(n) logger;
      /end-free
     P                 E


     /**
      * \brief Log message
      *
      * All message are written to the file FFPLOG.
      *
      * \param Logger
      * \param Message
      * \param Data
      * \param Message type
      */
     P ffp_logger_db_log...
     P                 B                   export
     D                 PI
     D   logger                        *   const
     D   message                  65535A   const varying
     D   data                     65535A   const varying options(*omit:*nopass)
     D   type                        10I 0 const options(*nopass)
      *
     D ffplog_out    E DS                  extname(FFPLOG : *OUTPUT)
     D                                     qualified
     D user            S             10A   inz(*user)
     D l_message       S          65535A   varying static
      /free
       if (not %open(FFPLOG));
         open FFPLOG;
       endif;

       if (%parms() <= 3);
         ffplog_out.loglevel = FFP_LOG_LEVEL_FATAL;
       else;
         ffplog_out.loglevel = type;
       endif;

       if (%parms() >= 3 and %addr(data) <> *null);
         ffplog_out.data = data;
       else;
         ffplog_out.data = '';
       endif;

       %len(l_message) = 0;
       l_message = message;
       ffplog_out.message = l_message;
       ffplog_out.createtime = %timestamp();
       ffplog_out.createdate = %date();
       ffplog_out.createuser = user;
       write FFPLOG.FFPLOGF00 ffplog_out;
      /end-free
     P                 E
