     /**
      * \brief Flat File Parser : Serializer Interface
      *
      * This serializer module stores data in a database file, FFPSER.
      *
      * \author Mihael Schmidt
      * \date   21.04.2010
      *
      * \implements FFPSER - Flat File Parser Serializer
      *
      * \todo not working atm
      * 
      */


     H nomain

      *-------------------------------------------------------------------------
      *
      * (C) Copyleft 2011 Mihael Schmidt
      *
      * This file is part of Flat File Parser (FFP).
      *
      * FFP is free software: you can redistribute it and/or modify
      * it under the terms of the GNU General Public License as published
      * the Free Software Foundation, either version 3 of the License, or
      * any later version.
      *
      * FFP is distributed in the hope that it will be useful,
      * but WITHOUT ANY WARRANTY; without even the implied warranty of
      * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      * GNU General Public License for more details.
      *
      * You should have received a copy of the GNU General Public License
      * along with FFP.  If not, see <http://www.gnu.org/licenses/>.
      *
      *-------------------------------------------------------------------------
      

      *-------------------------------------------------------------------------
      * Files
      *-------------------------------------------------------------------------
     FFFPSER    O    E             DISK    qualified usropn


      *-------------------------------------------------------------------------
      * Prototypes
      *-------------------------------------------------------------------------
     D getFreeTransactionId...
     D                 PR            20I 0
      *
      /copy 'database_serializer_h.rpgle'
      /copy 'serializer_t.rpgle'
      /copy 'llist_h.rpgle'
      /copy 'libc_h.rpgle'


      *-------------------------------------------------------------------------
      * Constants
      *-------------------------------------------------------------------------
     D SERIALIZER_ID...
     D                 C                   'ffp_serializer_db'


      *-------------------------------------------------------------------------
      * Procedures
      *-------------------------------------------------------------------------


     /**
      * \brief Create a serializer
      *
      * No user data are needed for the creation of the serializer instance.
      *
      * \param User data
      *
      * \return Serializer
      */
     P ffp_serializer_db_create...
     P                 B                   export
     D                 PI              *
     D   userdata                      *   const options(*nopass)
      *
     D header          DS                  likeds(ffp_serializer)
     D                                     based(serializer)
      /free
       serializer = %alloc(%size(ffp_serializer));

       clear header;

       header.id = SERIALIZER_ID;
       header.proc_serialize = %paddr('ffp_serializer_db_serialize');
       header.proc_finalize = %paddr('ffp_serializer_db_finalize');

       if (%parms() = 0 or userdata = *null);
         header.userdata = *null;
       else;
         header.userdata = %alloc(strlen(userdata) + 1); // speicher fr null reservieren
         memcpy(header.userdata : userdata : strlen(userdata) + 1); // null mit kopieren
       endif;

       return serializer;
      /end-free
     P                 E


     /**
      * \brief Free resources
      *
      * All previously allocated resources are freed.
      *
      * \param Serializer
      */
     P ffp_serializer_db_finalize...
     P                 B                   export
     D                 PI
     D  serializer                     *
      *
     D header          DS                  likeds(ffp_serializer)
     D                                     based(serializer)
      /free
       if (%open(FFPSER));
         close FFPSER;
       endif;

       if (header.userdata <> *null);
         dealloc header.userdata;
       endif;

       dealloc(n) serializer;
      /end-free
     P                 E


     /**
      * \brief Save transaction data
      *
      * All data will be saved in the transaction file.
      *
      * \param Serializer
      * \param Data
      *
      * \return Unique id for the saved transaction.
      */
     P ffp_serializer_db_serialize...
     P                 B                   export
     D                 PI          1000A   varying
     D   serializer                    *   const
     D   data                          *   const
      *
     D header          DS                  likeds(ffp_serializer)
     D                                     based(serializer)
      *
     D ffptran_out   E DS                  extname(FFPSER : *OUTPUT) qualified
     D id              S             20I 0
     D user            S             10A   inz(*user)
     D erstellung      S               Z
     D heute           S               D   inz(*sys)
     D lineEntry       DS                  likeds(ffp_processor_grouping_line)
     D                                     based(ptr)
      /free
       if (not %open(FFPSER));
         open FFPSER;
       endif;

       erstellung = %timestamp();
       id = getFreeTransactionId();

       // sicherstellen, dass auch von anfang an gelesen wird
       list_abortIteration(data);

       ptr = list_getNext(data);
       dow (ptr <> *null);
         ffptran_out.id = id;
         ffptran_out.counter = lineEntry.lineNumber;
         ffptran_out.data = lineEntry.data;
         ffptran_out.creattimst = erstellung;
         ffptran_out.createdate = heute;
         ffptran_out.createuser = user;
         write FFPSER.FFPSERF00 ffptran_out;

         ptr = list_getNext(data);
       enddo;

       return %char(id);
      /end-free
     P                 E


     /**
      * \brief Get next free transaction id
      *
      * \return transaction id
      */
     P getFreeTransactionId...
     P                 B
     D                 PI            20I 0
      *
     D id              S             20I 0
      /free
       exec sql SET :id = NEXT VALUE FOR ffpserdb;
       return id;
      /end-free
     P                 E
