     /**
      * \brief Flat File Parser : Utilities
      *
      * \author Mihael Schmidt
      * \date   07.06.2010
      *
      * \serviceprogram
      */


      *-------------------------------------------------------------------------
      *
      * (C) Copyleft 2011 Mihael Schmidt
      *
      * This file is part of Flat File Parser (FFP).
      *
      * FFP is free software: you can redistribute it and/or modify
      * it under the terms of the GNU General Public License as published
      * the Free Software Foundation, either version 3 of the License, or
      * any later version.
      *
      * FFP is distributed in the hope that it will be useful,
      * but WITHOUT ANY WARRANTY; without even the implied warranty of
      * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      * GNU General Public License for more details.
      *
      * You should have received a copy of the GNU General Public License
      * along with FFP.  If not, see <http://www.gnu.org/licenses/>.
      *
      *-------------------------------------------------------------------------


     H nomain


      *-------------------------------------------------------------------------
      * Prototypes
      *-------------------------------------------------------------------------
     D system_listJobLogMessages...
     D                 PR                  extpgm('QMHLJOBL')
     D  userspace                    20A   const
     D  format                        8A   const
     D  selection                    84A
     D  selectionSize                10I 0 const
     D  selFormat                     8A   const
     D  error                              likeds(QUSEC)
      *
      /copy 'parser_h.rpgle'
      /copy 'utilities_h.rpgle'
      /copy 'logger_h.rpgle'
      /copy 'message_h.rpgle'
      /copy 'llist_h.rpgle'
      /copy 'libc_h.rpgle'
      /copy 'ifsio_h.rpgle'
      /copy 'userspace_h.rpgle'


      *-------------------------------------------------------------------------
      * Templates
      *-------------------------------------------------------------------------
     D JSLT0100        DS                  qualified template
     D   maxMessages                 10I 0 inz(-1)
     D   listDirection...
     D                               10A   inz('*PRV')
     D   jobName                     10A   inz('*')
     D   userName                    10A
     D   jobNumber                    6A
     D   internalJobId...
     D                               16A
     D   messageKey                   4A   inz(x'FFFFFFFF')
     D   maxMessageLength...
     D                               10I 0 inz(-1)
     D   maxMessageHeaderLength...
     D                               10I 0
     D   offsetFieldReturned...
     D                               10I 0 inz(84)
     D   numberFieldsReturned...
     D                               10I 0 inz(1)
     D   offsetMessageQueue...
     D                               10I 0 inz(88)
     D   lengthMessageQueue...
     D                               10I 0 inz(4)
     D   field1                      10I 0 inz(0302)
     D   messageQueueName...
     D                                4A   inz('*')
      *
     D LJOB0100        DS                  qualified template
     D   nextEntryOffset...
     D                               10I 0
     D   fieldReturnedOffset...
     D                               10I 0
     D   numberFieldReturned...
     D                               10I 0
     D   messageSeverity...
     D                               10I 0
     D   messageId                    7A
     D   messageType                  2A
     D   messageKey                   4A
     D   messageFileName...
     D                               10A
     D   messageFileLibrary...
     D                               10A
     D   dateSent                     7A
     D   timeSent                     6A
     D   reserved                  8192A
      *
     D tmpl_messageEntry...
     D                 DS                  qualified template
     D   nextFieldOffset...
     D                               10I 0
     D   fieldInfoLength...
     D                               10I 0
     D   fieldId                     10I 0
     D   dataType                     1A
     D   dataStatus                   1A
     D   reserved2                   14A
     D   dataLength                  10I 0
     D   dataText                  1000A


      *-------------------------------------------------------------------------
      * Constants
      *-------------------------------------------------------------------------
     D CRLF            C                   x'0d25'


      *-------------------------------------------------------------------------
      * Globals
      *-------------------------------------------------------------------------
      /copy 'program_data_structure_h.rpgle'


      *-------------------------------------------------------------------------
      * Procedures
      *-------------------------------------------------------------------------


     /**
      * \brief Get the field data
      *
      * Returns the data for a specific field defined in the
      * line format. If the field value is 0 (Null), the entire
      * data string is returned.
      *
      * \author Mihael Schmidt
      * \date   25.01.2008
      *
      * \param Logger
      * \param Pointer to line format data structure
      * \param Pointer to data (null-terminated)
      * \param Field index
      *
      * \return Field data
      */
     P ffp_util_getLineFormatFieldData...
     P                 B                   export
     D                 PI         65535A   varying
     D   logger                        *   const
     D   lineFormatPtr...
     D                                 *   const
     D   data                     65535A   const varying
     D   field                       10I 0 const
      *
     D lineFormat      DS                  likeds(ffp_lineFormat)
     D                                     based(lineFormatPtr)
     D dataLength      S             10I 0
     D length          S             10I 0
     D retVal          S          65535A   varying
      /free
       if (lineFormatPtr = *null);
         if (logger <> *null);
           ffp_logger_log(logger : 'Prozedur ffp_util_getLineFormatFieldData ' +
                          'ist kein gltiges Zeileformat bergeben worden ' +
                          'fr die Daten ' + %trimr(data) + '.');
         endif;

         msg_sendEscapeMessage('Prozedur ffp_util_getLineFormatFieldData ' +
                               'ist kein gltiges Zeileformat bergeben ' +
                               'worden fr die Daten ' + %trimr(data) + '.');
       endif;

       monitor;
         dataLength = %len(data) - lineFormat.fields(field).start + 1;

         if (dataLength < lineFormat.fields(field).length);
           length = dataLength;
         else;
           length = lineFormat.fields(field).length;
         endif;

         retVal = %subst(data : lineFormat.fields(field).start : length);
         on-error *all;

           if (logger <> *null);
             ffp_logger_log(logger : 'Feld ' + %char(field) + ' aus den Daten '+
                            %trimr(data) + ' konnte nicht ermittelt werden. ' +
                            'Zeilenformat: ' + %trimr(lineFormat.name) + '.');
           endif;

           msg_sendEscapeMessage('Fr die Daten ' + data + ' kann nicht das ' +
                                 %char(field) + '. Feld ausgewertet werden. ' +
                                 'Systemfehler: ' + programStatusDs.messageId +
                                 ': ' + ffp_util_getLastErrorMessage(
                                    programStatusDs.messageId));
       endmon;
       return retVal;
      /end-free
     P                 E


     /**
      * \brief Letzte Fehlernachricht holen
      *
      * Gibt den Fehlertext zurck aus der letzten Nachricht im Job Log, die der
      * bergebenen Nachrichten-Id entspricht (zum Beispiel: CPF9898).
      *
      * \param Nachrichten-Id
      *
      * \return Nachrichtentext oder leer, falls keine Nachricht mit der bergebenen
      *         Nachrichten-Id im Job Log
      */
     P ffp_util_getLastErrorMessage...
     P                 B                   export
     D                 PI          1024A   varying
     D   messageId                    7A   const
      *
     D message         S           1024A   varying
     D usName          S             10A
     D usLib           S             10A
     D usHeader        DS                  likeds(tmpl_userspace_header)
     D                                     based(usPtr)
     D userspace_attr  DS                  likeds(tmpl_userspace_attributes) 
     D i               S             10I 0
     D l_JSLT0100      DS                  likeds(JSLT0100) inz
     D l_LJOB0100      DS                  likeds(LJOB0100) based(listPtr)
     D messageEntryDs...
     D                 DS                  likeds(tmpl_messageEntry)
     D                                     based(entryPtr)
      /free
       clear QUSEC;
       QUSBPRV = 0;
       usLib = 'QTEMP';
       usName = %subst(%str(tmpnam(*omit)) : 7);

       userspace_create(usName + usLib : *blank : 4096 : X'00' : '*ALL' :
                         'FFP Util UserSpace' : '*YES' : QUSEC );

       // change user space attributes to autoextend
       userspace_attr.size = 1;       // 1=attribute will be changed
       userspace_attr.key = 3;        // 3=extensibility attribute
       userspace_attr.dataLength = 1;
       userspace_attr.data = '1';     // user space is extensible
       userspace_changeAttributes(usLib : usName + usLib :userspace_attr:QUSEC);

       userspace_retrievePointer(usName + usLib : usPtr : QUSEC);
      
       clear l_JSLT0100;
       l_JSLT0100.maxMessages = -1;
       l_JSLT0100.listDirection = '*PRV';
       l_JSLT0100.jobName = '*';
       l_JSLT0100.messageKey = x'FFFFFFFF';
       l_JSLT0100.maxMessageLength = -1;
       l_JSLT0100.offsetFieldReturned = 84;
       l_JSLT0100.numberFieldsReturned = 1;
       l_JSLT0100.offsetMessageQueue = 88;
       l_JSLT0100.lengthMessageQueue = 4;
       l_JSLT0100.field1 = 0302;
       l_JSLT0100.messageQueueName = '*';

       clear QUSEC;
       QUSBPRV = 0;
       system_listJobLogMessages(usName + usLib : 'LJOB0100' : l_JSLT0100 :
                                 %len(l_JSLT0100) : 'JSLT0100' : QUSEC);

       listPtr = usPtr + usHeader.offsetList;
       entryPtr = usPtr + l_LJOB0100.fieldReturnedOffset;

       for i = 1 to usHeader.nmbrEntries-1;
         if (l_LJOB0100.messageId = messageId);
           message = %subst(messageEntryDs.dataText : 1 :
                            messageEntryDs.dataLength);
           leave;
         endif;

         listPtr = usPtr + l_LJOB0100.nextEntryOffset;
         entryPtr = usPtr + l_LJOB0100.fieldReturnedOffset;
       endfor;

       userspace_delete(usName + usLib : QUSEC);

       return message;
      /end-free
     P                 E


     /**
      * \brief Dump collected data to stream file
      *
      * Dumps all lines of data to the passed stream file. If the stream file
      * already exists the data will be appended else the file will be created
      * with the CCSID of the job or the passed CCSID if the third parameter
      * was passed.
      *
      * \param File path
      * \param Data
      * \param CCSID (optional)
      * \param Delete existing file (optional)
      *
      *  \throws CPF9898 Could not create/write to stream file
      */
     P ffp_util_dumpDataToFile...
     P                 B                   export
     D                 PI
     D   filePath                  1024A   const
     D   data                          *   const
     D   pCcsid                      10I 0 const options(*nopass : *omit)
     D   deleteFile                    N   const options(*nopass)
      *
     D ccsid           S             10I 0
     D fileHandle      S             10I 0
     D flags           S             10I 0
     D mode            S             10I 0
     D ptr             S               *
     D errorNumber     S             10I 0 based(errorPtr)
     D errorString     S           1024A   varying

      /free
       if (%parms() >= 3 and %addr(pCcsid) <> *null);
         ccsid = pCcsid;
       endif;

       if (%parms() = 4 and deleteFile);
         if (access(%trimr(filePath) : F_OK) >= 0);
           // file exist => delete
           system('RMVLNK OBJLNK(''' + %trimr(filePath) + ''')');
         endif;
       endif;


       // create file
       flags = O_WRONLY + O_CREAT + O_APPEND + O_TEXTDATA +
               O_CCSID + O_TEXT_CREAT;
       mode =  S_IRUSR + S_IWUSR + S_IRGRP + S_IROTH;
       fileHandle = open(%trimr(filePath) : flags : mode : ccsid : 0);
       if (fileHandle < 0);
         // could not create file
         errorPtr = errno();
         errorString = 'Could not create file ' + %trimr(filePath) + ': ' +
                       %str(strerr(errorNumber));
         msg_sendEscapeMessage(errorString);
       endif;

       // write content
       list_abortIteration(data);
       ptr = list_getNext(data);
       dow (ptr <> *null);
         callp write(fileHandle : ptr : strlen(ptr));

         ptr = list_getNext(data);
       enddo;

       // close file
       callp close(fileHandle);

      /end-free
     P                 E
